"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getHighlighter = getHighlighter;
exports.getSafeLanguage = getSafeLanguage;
exports.isValidLanguage = isValidLanguage;
var _core = require("@shikijs/core");
var _engineJavascript = require("@shikijs/engine-javascript");
var _css = _interopRequireDefault(require("shiki/dist/langs/css.mjs"));
var _html = _interopRequireDefault(require("shiki/dist/langs/html.mjs"));
var _javascript = _interopRequireDefault(require("shiki/dist/langs/javascript.mjs"));
var _json = _interopRequireDefault(require("shiki/dist/langs/json.mjs"));
var _markdown = _interopRequireDefault(require("shiki/dist/langs/markdown.mjs"));
var _php = _interopRequireDefault(require("shiki/dist/langs/php.mjs"));
var _tsx = _interopRequireDefault(require("shiki/dist/langs/tsx.mjs"));
var _typescript = _interopRequireDefault(require("shiki/dist/langs/typescript.mjs"));
var _githubDark = _interopRequireDefault(require("shiki/dist/themes/github-dark.mjs"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
/* eslint-disable no-console */

// Import languages with a more explicit structure

const DEBUG = true;
function log(message, ...args) {
  if (DEBUG) {
    console.log(`[ShikiContext] ${message}`, ...args);
  }
}
// Process raw grammar into a proper language registration
function processGrammar(grammar, id) {
  if (!grammar) {
    log(`Warning: Grammar for ${id} is null`);
    return createPlainTextGrammar(id);
  }

  // Ensure required properties exist
  const processed = {
    ...grammar,
    name: id,
    scopeName: grammar.scopeName || `source.${id}`,
    displayName: grammar.displayName || id,
    aliases: grammar.aliases || [],
    embeddedLangs: grammar.embeddedLanguages || [],
    patterns: grammar.patterns || [],
    repository: grammar.repository || {}
  };
  log(`Processed grammar for ${id}:`, {
    scopeName: processed.scopeName,
    patterns: processed.patterns.length,
    repository: Object.keys(processed.repository).length
  });
  return processed;
}

// Create a simple plaintext grammar
function createPlainTextGrammar(id) {
  return {
    name: id,
    scopeName: `source.${id}`,
    repository: {},
    patterns: [{
      match: '.*',
      name: 'text'
    }]
  };
}

// Map raw grammars to processed language registrations
const languageRegistrations = {
  typescript: processGrammar(_typescript.default, 'typescript'),
  tsx: processGrammar(_tsx.default, 'tsx'),
  javascript: processGrammar(_javascript.default, 'javascript'),
  json: processGrammar(_json.default, 'json'),
  html: processGrammar(_html.default, 'html'),
  css: processGrammar(_css.default, 'css'),
  markdown: processGrammar(_markdown.default, 'markdown'),
  php: processGrammar(_php.default, 'php'),
  plaintext: createPlainTextGrammar('plaintext')
};
let highlighterInstance = null;
async function getHighlighter() {
  if (highlighterInstance) {
    log('Returning cached highlighter instance');
    return highlighterInstance;
  }
  try {
    log('Creating new highlighter instance...');

    // Initialize the regex engine with less complex patterns
    const engine = (0, _engineJavascript.createJavaScriptRegexEngine)();

    // Load and verify language registrations
    log('Verifying language registrations...');
    const validRegistrations = Object.entries(languageRegistrations).filter(([id, reg]) => {
      const isValid = reg.scopeName && Array.isArray(reg.patterns);
      if (!isValid) {
        log(`Warning: Invalid registration for ${id}`);
      }
      return isValid;
    }).map(([, reg]) => reg);
    log(`Valid registrations: ${validRegistrations.length}`);

    // Create highlighter with verified registrations
    highlighterInstance = await (0, _core.createHighlighterCore)({
      themes: [_githubDark.default],
      langs: validRegistrations,
      engine
    });
    const loadedLangs = highlighterInstance.getLoadedLanguages();
    log('Successfully loaded languages:', loadedLangs);
    return highlighterInstance;
  } catch (error) {
    log('Error creating highlighter:', error);
    throw error;
  }
}

// Helper functions
function isValidLanguage(lang) {
  return lang in languageRegistrations;
}
function getSafeLanguage(lang) {
  const normalized = lang.toLowerCase();
  if (isValidLanguage(normalized)) {
    return normalized;
  }
  log(`Warning: Unsupported language '${lang}', falling back to plaintext`);
  return 'plaintext';
}
//# sourceMappingURL=SyntaxContext.js.map