import type { LanguageRegistration as ILanguageRegistration } from '@shikijs/types'
/* eslint-disable no-console */
import { createHighlighterCore, type HighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'

import cssGrammar from 'shiki/dist/langs/css.mjs'
import htmlGrammar from 'shiki/dist/langs/html.mjs'
import javascriptGrammar from 'shiki/dist/langs/javascript.mjs'
import jsonGrammar from 'shiki/dist/langs/json.mjs'
import markdownGrammar from 'shiki/dist/langs/markdown.mjs'
import phpGrammar from 'shiki/dist/langs/php.mjs'
// Import languages with a more explicit structure
import tsxGrammar from 'shiki/dist/langs/tsx.mjs'
import typescriptGrammar from 'shiki/dist/langs/typescript.mjs'
import githubDark from 'shiki/dist/themes/github-dark.mjs'

const DEBUG = true
function log(message: string, ...args: any[]) {
  if (DEBUG) {
    console.log(`[ShikiContext] ${message}`, ...args)
  }
}

export type AvailableLanguage = 'php' | 'tsx' | 'typescript' | 'javascript' | 'json' | 'html' | 'css' | 'markdown' | 'plaintext'

// Process raw grammar into a proper language registration
function processGrammar(grammar: any, id: string): ILanguageRegistration {
  if (!grammar) {
    log(`Warning: Grammar for ${id} is null`)
    return createPlainTextGrammar(id)
  }

  // Ensure required properties exist
  const processed = {
    ...grammar,
    name: id,
    scopeName: grammar.scopeName || `source.${id}`,
    displayName: grammar.displayName || id,
    aliases: grammar.aliases || [],
    embeddedLangs: grammar.embeddedLanguages || [],
    patterns: grammar.patterns || [],
    repository: grammar.repository || {},
  }

  log(`Processed grammar for ${id}:`, {
    scopeName: processed.scopeName,
    patterns: processed.patterns.length,
    repository: Object.keys(processed.repository).length,
  })

  return processed
}

// Create a simple plaintext grammar
function createPlainTextGrammar(id: string): ILanguageRegistration {
  return {
    name: id,
    scopeName: `source.${id}`,
    repository: {},
    patterns: [
      {
        match: '.*',
        name: 'text',
      },
    ],
  }
}

// Map raw grammars to processed language registrations
const languageRegistrations: Record<AvailableLanguage, ILanguageRegistration> = {
  typescript: processGrammar(typescriptGrammar, 'typescript'),
  tsx: processGrammar(tsxGrammar, 'tsx'),
  javascript: processGrammar(javascriptGrammar, 'javascript'),
  json: processGrammar(jsonGrammar, 'json'),
  html: processGrammar(htmlGrammar, 'html'),
  css: processGrammar(cssGrammar, 'css'),
  markdown: processGrammar(markdownGrammar, 'markdown'),
  php: processGrammar(phpGrammar, 'php'),
  plaintext: createPlainTextGrammar('plaintext'),
}

let highlighterInstance: HighlighterCore | null = null

export async function getHighlighter() {
  if (highlighterInstance) {
    log('Returning cached highlighter instance')
    return highlighterInstance
  }

  try {
    log('Creating new highlighter instance...')

    // Initialize the regex engine with less complex patterns
    const engine = createJavaScriptRegexEngine()

    // Load and verify language registrations
    log('Verifying language registrations...')
    const validRegistrations = Object.entries(languageRegistrations)
      .filter(([id, reg]) => {
        const isValid = reg.scopeName && Array.isArray(reg.patterns)
        if (!isValid) {
          log(`Warning: Invalid registration for ${id}`)
        }
        return isValid
      })
      .map(([, reg]) => reg)

    log(`Valid registrations: ${validRegistrations.length}`)

    // Create highlighter with verified registrations
    highlighterInstance = await createHighlighterCore({
      themes: [githubDark],
      langs: validRegistrations,
      engine,
    })

    const loadedLangs = highlighterInstance.getLoadedLanguages()
    log('Successfully loaded languages:', loadedLangs)

    return highlighterInstance
  }
  catch (error) {
    log('Error creating highlighter:', error)
    throw error
  }
}

// Helper functions
export function isValidLanguage(lang: string): lang is AvailableLanguage {
  return lang in languageRegistrations
}

export function getSafeLanguage(lang: string): AvailableLanguage {
  const normalized = lang.toLowerCase()
  if (isValidLanguage(normalized)) {
    return normalized as AvailableLanguage
  }
  log(`Warning: Unsupported language '${lang}', falling back to plaintext`)
  return 'plaintext'
}
